(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.aimless = {})));
}(this, (function (exports) { 'use strict';

var defaultEngine = function defaultEngine() {
  try {
    // Credit @ Tamás Sallai
    var crypto = window.crypto || window.msCrypto;
    var buffer = new ArrayBuffer(8);
    var ints = new Int8Array(buffer);
    crypto.getRandomValues(ints);
    ints[7] = 63;
    ints[6] |= 0xf0;
    return new DataView(buffer).getFloat64(0, true) - 1;
  } catch (e) {
    return Math.random();
  }
};
var generateListFromRange = function generateListFromRange(min, max) {
  var result = [];
  for (var i = min; i <= max; i++) {
    result.push(i);
  }
  return result;
};
var randIntRange = function randIntRange(i, j) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  var min = Math.ceil(i);
  var max = Math.floor(j);
  return Math.floor(engine() * (max - min + 1)) + min;
};
var sliceOut = function sliceOut(arr, i) {
  return arr.slice(0, i).concat(arr.slice(i + 1));
};
var generateCurry = function generateCurry(func) {
  return function () {
    var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      return func.apply(void 0, args.concat([engine]));
    };
  };
};

/**
 * Returns a random number within the integer range
 * @param min Minimum integer
 * @param max Maximum integer
 * @param engine PRNG of choice
 */
var intRange = function intRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return randIntRange(min, max, engine);
};
var intRangeWithEngine = generateCurry(intRange);

/**
 * Returns a random boolean (true or false)
 * @param engine PRNG of choice
 */
var bool = function bool() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return !!intRange(0, 1, engine);
};
var boolWithEngine = generateCurry(bool);

/**
 * Returns a random character from the provided array
 * @param str String
 * @param engine PRNG of choice
 */
var char = function char(str) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return str[intRange(0, str.length - 1, engine)];
};
var charWithEngine = generateCurry(char);

/**
 * Returns a random value that follows a custom distribution
 * @param func Distribution function that accepts a random number between 0 and 1
 * @param engine PRNG of choice
 */
var customDist = function customDist(func) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return func(engine());
};
var customDistWithEngine = generateCurry(customDist);

/**
 * Returns a random value that follows an exponential distribution
 * @param lambda Lambda
 * @param engine PRNG of choice
 */
var exponentialDist = function exponentialDist(lambda) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return -Math.log(1 - engine()) / lambda;
};
var exponentialDistWithEngine = generateCurry(exponentialDist);

/**
 * Returns a random number within the float range
 * @param min Minimum value
 * @param max Maximum value
 * @param engine PRNG of choice
 */
var floatRange = function floatRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return engine() * (max - min) + min;
};
var floatRangeWithEngine = generateCurry(floatRange);

/**
 * Returns a new array with randomized order
 * @param arr Array of values
 * @param engine PRNG of choice
 */
var sequence = function sequence(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  var result = [];
  var tempArr = arr;
  var i;
  while (result.length < arr.length) {
    i = intRange(0, tempArr.length - 1, engine);
    result.push(tempArr[i]);
    tempArr = sliceOut(tempArr, i);
  }
  return result;
};
var sequenceWithEngine = generateCurry(sequence);

/**
 * Returns an array with every value in the range in random order
 * @param min Minimum value
 * @param max Maximum value
 * @param engine PRNG of choice
 */
var intSequence = function intSequence(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return sequence(generateListFromRange(min, max), engine);
};
var intSequenceWithEngine = generateCurry(intSequence);

/**
 * Returns a random value that follows a normal distribution
 * @param mean Mean
 * @param stdDev Standard Deviation
 * @param engine PRNG of choice
 */
var normalDist = function normalDist(mean, stdDev) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  // Box-Muller transform
  var u, v, s;
  do {
    u = engine() * 2 - 1;
    v = engine() * 2 - 1;
    s = Math.pow(u, 2) + Math.pow(v, 2);
  } while (s === 0 || s >= 1);
  var t = Math.sqrt(-2 * Math.log(s) / s);
  var rand = mean + stdDev * u * t;
  return rand;
};
var normalDistWithEngine = generateCurry(normalDist);

/**
 * Returns a random number between -1 and 1
 * @param engine PRNG of choice
 */
var normalFloat = function normalFloat() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return floatRange(-1, 1, engine);
};
var normalFloatWithEngine = generateCurry(normalFloat);

/**
 * Returns a random value from the provided array
 * @param arr Array of values
 * @param engine PRNG of choice
 */
var oneOf = function oneOf(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return arr[intRange(0, arr.length - 1, engine)];
};
var oneOfWithEngine = generateCurry(oneOf);

/**
 * Returns a seeded PRNG function
 * @param seed Seed (integer)
 */
var seedFunc = function seedFunc(seed) {
  // Park-Miller PRNG
  var currentSeed = seed % 2147483647;
  return function () {
    currentSeed = currentSeed * 16807 % 2147483647;
    return (currentSeed - 1) / 2147483646;
  };
};

/**
 * Returns a random sign (-1 or 1)
 * @param engine PRNG of choice
 */
var sign = function sign() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return bool(engine) ? 1 : -1;
};
var signWithEngine = generateCurry(sign);

/**
 * Returns a PRNG function that returns unique values from the provided array.
 * @param arr Array of values to pull from
 * @param engine Engine to be used as PRNG
 */
var uniqFuncSequence = function uniqFuncSequence(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  var tempArr = arr;
  return function () {
    if (!tempArr.length) return null;
    var i = randIntRange(0, tempArr.length - 1, engine);
    var result = tempArr[i];
    tempArr = sliceOut(tempArr, i);
    return result;
  };
};

/**
 * Returns a PRNG function that returns unique values from the provided range.
 * @param min Minimum integer
 * @param max Maximum integer
 * @param engine Engine to be used as PRNG
 */
var uniqFuncIntRange = function uniqFuncIntRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return uniqFuncSequence(generateListFromRange(min, max), engine);
};

/**
 * Returns a valid RFC4122 version4 ID hex string, using the provided engine
 * @param engine PRNG of choice
 */
var uuid = function uuid() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  // Credit @Alexey Silin from https://gist.github.com/1308368
  var a = '';
  var b = '';
  for (b = a = ''; a++ < 36; b += ~a % 5 | a * 3 & 4 ? (a ^ 15 ? 8 ^ engine() * (a ^ 20 ? 16 : 4) : 4).toString(16) : '-') {}
  return b;
};
var uuidWithEngine = generateCurry(uuid);

/**
 * Returns a random value from the provided array, biased towards provided weights
 * @param nums Array of numbers (integer or float)
 * @param weights Array of weights (integer or float)
 * @param engine PRNG of choice
 */
var weighted = function weighted(nums, weights) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  if (nums.length !== weights.length) {
    throw new Error('Every provided number must have a corresponding weight.');
  }
  var totalWeight = weights.reduce(function (sum, weight) {
    return sum + weight;
  }, 0);
  var rand = engine() * totalWeight;
  var cumulativeWeight = 0;
  var selectedIndex = 0;
  for (var i = 0; i < nums.length; i++) {
    cumulativeWeight += weights[i];
    if (rand < cumulativeWeight) {
      selectedIndex = i;
      break;
    }
  }
  return nums[selectedIndex];
};
var weightWithEngine = generateCurry(weighted);

exports.bool = bool;
exports.boolWithEngine = boolWithEngine;
exports.char = char;
exports.charWithEngine = charWithEngine;
exports.customDist = customDist;
exports.customDistWithEngine = customDistWithEngine;
exports.exponentialDist = exponentialDist;
exports.exponentialDistWithEngine = exponentialDistWithEngine;
exports.floatRange = floatRange;
exports.floatRangeWithEngine = floatRangeWithEngine;
exports.intRange = intRange;
exports.intRangeWithEngine = intRangeWithEngine;
exports.intSequence = intSequence;
exports.intSequenceWithEngine = intSequenceWithEngine;
exports.normalDist = normalDist;
exports.normalDistWithEngine = normalDistWithEngine;
exports.normalFloat = normalFloat;
exports.normalFloatWithEngine = normalFloatWithEngine;
exports.oneOf = oneOf;
exports.oneOfWithEngine = oneOfWithEngine;
exports.seedFunc = seedFunc;
exports.sequence = sequence;
exports.sequenceWithEngine = sequenceWithEngine;
exports.sign = sign;
exports.signWithEngine = signWithEngine;
exports.uniqFuncIntRange = uniqFuncIntRange;
exports.uniqFuncSequence = uniqFuncSequence;
exports.uuid = uuid;
exports.uuidWithEngine = uuidWithEngine;
exports.weighted = weighted;
exports.weightWithEngine = weightWithEngine;

Object.defineProperty(exports, '__esModule', { value: true });

})));
